@@ -132,6 +132,13 @@ gboolean            visit_rc_directory           (const gchar *rc_dir,
 typedef void (*VisitFunc) (gchar *real_file, gchar *rc_file, gpointer data);
 typedef void (*VisitDoneFunc) (const GList *dir_list, gpointer data);
 
+gboolean            visit_rc_directory_sync      (const gchar *rc_dir,
+						  const gchar *rc_ext,
+						  const char *dir,
+						  gboolean recursive,
+						  VisitFunc do_something,
+						  gpointer data);
+
 void                visit_rc_directory_async     (const gchar *rc_dir,
 						  const gchar *rc_ext,
 						  const char *dir,
@@ -181,5 +188,6 @@ GnomeVFSFileSize    get_dest_free_space          (const char  *path);
 
 gboolean            is_mime_type_writable        (const char *mime_type);
 
+int	            check_permissions            (const char *path, int mode);
 
 #endif /* FILE_UTILS_H */
