@@ -244,7 +244,13 @@ thumb_loader (const char  *path,
 		if (pixbuf != NULL) {
 			animation = gdk_pixbuf_non_anim_new (pixbuf);
 			g_object_unref (pixbuf);
-		}
+
+			if (animation == NULL)
+				g_print ("ANIMATION == NULL\n");
+
+		} else
+			g_print ("PIXBUF == NULL\n");
+
 	} else
 #endif
 		animation = gdk_pixbuf_animation_new_from_file (path, error);
@@ -345,7 +351,7 @@ thumb_loader_set_path (ThumbLoader *tl,
 	g_free (priv->uri);
 	g_free (priv->path);
 
-	escaped_path = gnome_vfs_escape_path_string (path);
+	escaped_path = gnome_vfs_escape_host_and_path_string (path);
 	vfs_uri = gnome_vfs_uri_new (escaped_path);
 	g_free (escaped_path);
 
@@ -394,7 +400,7 @@ thumb_loader_get_uri (ThumbLoader *tl)
 
 	priv = tl->priv;
 
-	escaped_path = gnome_vfs_escape_path_string (priv->path);
+	escaped_path = gnome_vfs_escape_host_and_path_string (priv->path);
 	uri = gnome_vfs_uri_new (escaped_path);
 	g_free (escaped_path);
 
