@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2004 The Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -146,9 +146,9 @@ list_view_button_press_cb (GdkEventButton     *event,
                             -1);
 
 	if (bookmarks)
-		utf8_folder_path = g_locale_to_utf8 (folder_path + FILE_PREFIX_L, -1, 0, 0, 0);
+		utf8_folder_path = g_filename_to_utf8 (folder_path + FILE_PREFIX_L, -1, 0, 0, 0);
 	else
-		utf8_folder_path = g_locale_to_utf8 (folder_path, -1, 0, 0, 0);
+		utf8_folder_path = g_filename_to_utf8 (folder_path, -1, 0, 0, 0);
 
 	gth_folder_selection_set_folder (GTH_FOLDER_SELECTION (folder_sel),
 					 utf8_folder_path);
@@ -209,9 +209,9 @@ list_view_activated_cb (GtkTreePath        *path,
 			    -1);
 	
 	if (bookmarks)
-		utf8_folder_path = g_locale_to_utf8 (folder_path + FILE_PREFIX_L, -1, 0, 0, 0);
+		utf8_folder_path = g_filename_to_utf8 (folder_path + FILE_PREFIX_L, -1, 0, 0, 0);
 	else
-		utf8_folder_path = g_locale_to_utf8 (folder_path, -1, 0, 0, 0);
+		utf8_folder_path = g_filename_to_utf8 (folder_path, -1, 0, 0, 0);
 
 	gth_folder_selection_set_folder (GTH_FOLDER_SELECTION (folder_sel),
 					 utf8_folder_path);
@@ -246,23 +246,25 @@ recent_activated_cb (GtkTreeView       *tree_view,
 
 
 static void
-file_sel_ok_clicked_cb (GObject  *object,
-			gpointer  data)
+file_sel_ok_clicked_cb (GtkDialog  *file_sel,
+			int         button_number,
+			gpointer    data)
 {
-	GtkWidget          *file_sel = data;
 	GthFolderSelection *folder_sel;
 	char               *folder;
 
+	if (button_number != GTK_RESPONSE_ACCEPT) {
+		gtk_widget_destroy (GTK_WIDGET (file_sel));
+		return;
+	}
+
 	folder_sel = g_object_get_data (G_OBJECT (file_sel), "folder_sel");
 	
-	folder = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel)));
-	gtk_widget_destroy (file_sel);
-
-	if (folder == NULL) 
-		return;
-
+	folder = g_strdup (gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel)));
 	gth_folder_selection_set_folder (folder_sel, folder);
 
+	gtk_widget_destroy (GTK_WIDGET (file_sel));
+
 	g_free (folder);
 }
 
@@ -277,12 +279,16 @@ browse_button_clicked_cb (GtkWidget *widget,
 	const char  *utf8_folder;
 	char        *folder;
 	
-	file_sel = gtk_file_selection_new (folder_sel->priv->title);
+	file_sel = gtk_file_chooser_dialog_new (folder_sel->priv->title, NULL,
+						GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+						NULL);
 	
 	entry = folder_sel->priv->file_entry;
 
 	utf8_folder = gtk_entry_get_text (GTK_ENTRY (entry));
-	folder = g_locale_from_utf8 (utf8_folder, -1, 0, 0, 0);
+	folder = g_filename_from_utf8 (utf8_folder, -1, 0, 0, 0);
 	if (folder[strlen (folder) - 1] != '/') {
 		char *tmp;
 		tmp = g_strconcat (folder, "/", NULL);
@@ -290,22 +296,16 @@ browse_button_clicked_cb (GtkWidget *widget,
 		folder = tmp;
 	}
 
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel), folder);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_sel), folder);
 	g_free (folder);
 	
 	g_object_set_data (G_OBJECT (file_sel), "folder_sel", folder_sel);
 
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-			  "clicked", 
+	g_signal_connect (G_OBJECT (GTK_DIALOG (file_sel)),
+			  "response", 
 			  G_CALLBACK (file_sel_ok_clicked_cb), 
-			  file_sel);
+			  folder_sel);
 	
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (file_sel));
-
-	gtk_widget_set_sensitive (GTK_WIDGET (GTK_FILE_SELECTION (file_sel)->file_list)->parent, FALSE);
 	gtk_window_set_transient_for (GTK_WINDOW (file_sel), 
 				      GTK_WINDOW (folder_sel));
 	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
@@ -558,11 +558,14 @@ void
 gth_folder_selection_set_folder (GthFolderSelection *fsel,
 				 const char         *folder)
 {
-	_gtk_entry_set_locale_text (GTK_ENTRY (fsel->priv->file_entry), folder);}
+	if ((folder == NULL) || (fsel == NULL)) 
+		return;
+	_gtk_entry_set_filename_text (GTK_ENTRY (fsel->priv->file_entry), folder);
+}
 
 
 char *
 gth_folder_selection_get_folder (GthFolderSelection *fsel)
 {
-	return _gtk_entry_get_locale_text (GTK_ENTRY (fsel->priv->file_entry));
+	return _gtk_entry_get_filename_text (GTK_ENTRY (fsel->priv->file_entry));
 }
