@@ -36,6 +36,10 @@
 #include "gthumb-preloader.h"
 #include "dlg-save-image.h"
 
+#ifdef HAVE_LIBEXIF
+#include <libexif/exif-data.h>
+#endif /* HAVE_LIBEXIF */
+
 #define GCONF_NOTIFICATIONS 18
 
 
@@ -61,6 +65,17 @@ typedef struct {
 
 	GtkWidget          *app;                /* The main window. */
 	BonoboUIComponent  *ui_component;
+	GtkUIManager       *ui;
+	GtkActionGroup     *actions;
+	GtkActionGroup     *bookmark_actions;
+	GtkActionGroup     *history_actions;
+	guint               sidebar_merge_id;
+	guint               toolbar_merge_id;
+	guint               bookmarks_merge_id;
+	guint               history_merge_id;
+
+	GtkWidget          *toolbar;
+	GtkWidget          *statusbar;
 
 	GtkWidget          *viewer;
 	GtkWidget          *viewer_container;  /* Container widget for the 
@@ -71,15 +86,29 @@ typedef struct {
 	GtkWidget          *content_pane;
 	GtkWidget          *image_pane;
 	GtkWidget          *dir_list_pane;
+	GtkWidget          *file_list_pane;
 	GtkWidget          *notebook;
 	GtkWidget          *location_entry;
 	GtkWidget          *viewer_vscr;
 	GtkWidget          *viewer_hscr;
 	GtkWidget          *viewer_event_box;
 	GtkWidget          *go_back_toolbar_button;
+	GtkWidget          *go_fwd_toolbar_button;
+	GtkWidget          *go_up_toolbar_button;
+	GtkWidget          *go_home_toolbar_button;
 	GtkWidget          *show_folders_toolbar_button;
 	GtkWidget          *show_catalog_toolbar_button;
-	GtkWidget          *popup_menu;
+
+	GtkWidget          *file_popup_menu;
+	GtkWidget          *image_popup_menu;
+	GtkWidget          *fullscreen_image_popup_menu;
+	GtkWidget          *catalog_popup_menu;
+	GtkWidget          *library_popup_menu;
+	GtkWidget          *dir_popup_menu;
+	GtkWidget          *dir_list_popup_menu;
+	GtkWidget          *catalog_list_popup_menu;
+	GtkWidget          *history_list_popup_menu;
+
 
 	GtkWidget          *image_comment;
 	GtkWidget          *exif_data_viewer;
@@ -91,7 +120,7 @@ typedef struct {
 	GtkWidget          *zoom_info_frame;
 
 	GtkWidget          *image_prop_dlg;        /* no-modal dialogs. */
-	GtkWidget          *comments_dlg;
+	GtkWidget          *comment_dlg;
 	GtkWidget          *categories_dlg;
 	GtkWidget          *bookmarks_dlg;
 
@@ -189,6 +218,10 @@ typedef struct {
 	GList              *slideshow_first;
 	GList              *slideshow_current;
 
+#ifdef HAVE_LIBEXIF
+	ExifData           *exif_data;
+#endif /* HAVE_LIBEXIF */
+
 	/* monitor stuff */
 
 	GnomeVFSMonitorHandle *monitor_handle;
@@ -208,6 +241,8 @@ typedef struct {
 	guint                  progress_timeout;
 
 	GtkTooltips           *tooltips;
+	guint                  help_message_cid;
+	guint                  list_info_cid;
 
 	gboolean               focus_location_entry;
 } GThumbWindow;
@@ -228,6 +263,10 @@ void            window_hide_image_pane              (GThumbWindow *window);
 
 void            window_show_image_pane              (GThumbWindow *window);
 
+void            window_hide_image_data              (GThumbWindow *window);
+
+void            window_show_image_data              (GThumbWindow *window);
+
 void            window_set_preview_content          (GThumbWindow      *window,
 						     GthPreviewContent  content);
 
@@ -275,6 +314,10 @@ void            window_stop_slideshow               (GThumbWindow *window);
 
 void            window_show_image_prop              (GThumbWindow *window);
 
+void            window_show_comment_dlg             (GThumbWindow *window);
+
+void            window_show_categories_dlg          (GThumbWindow *window);
+
 void            window_image_set_modified           (GThumbWindow *window,
 						     gboolean      modified);
 
